import pytest
from pytmg import TMGResult


def test_tmg_result_init():
    result_test_inputdata = {
        "totalCount": 1,
        "itemPerPage": None,
        "page": None,
        "filters": [],  # Intentionally making this an empty list, as it contains a lot of data we don't care about
        "networkDevices": [
            {
                "productFamily": "N9300",
                "networkFamilyDataSheet": "https://www.cisco.com/c/en/us/products/collateral/switches/nexus-9000-series-switches/datasheet-c78-741560.html",
                "networkAndTransceiverCompatibility": [
                    {
                        "productId": "N9K-C93180YC-EX",
                        "transceivers": [
                            {
                                "tmgId": 24739,
                                "productFamilyId": 5,
                                "productFamily": "QSFP100",
                                "productModelId": 1,
                                "productId": "QSFP-100G-SR4-S",
                                "version": " ",
                                "versionId": None,
                                "description": None,
                                "formFactor": "QSFP28",
                                "reach": "100m (OM4)",
                                "temperatureRange": "0 to 70C",
                                "digitalDiagnostic": "Y",
                                "cableType": "Ribbon Fiber",
                                "media": "MMF",
                                "connectorType": "MPO-12",
                                "transmissionStandard": " ",
                                "transceiverModelDataSheet": "https://www.cisco.com/c/en/us/products/collateral/interfaces-modules/transceiver-modules/datasheet-c78-736282.html",
                                "endOfSale": " ",
                                "dataRate": "100 Gbps",
                                "transceiverNotes": None,
                                "noteCount": 0,
                                "state": None,
                                "stateMessage": None,
                                "updatedOn": None,
                                "updatedBy": None,
                                "transceiverBusinessUnit": "TMG",
                                "networkModelId": 43,
                                "breakoutMode": " ",
                                "osType": "ACI",
                                "domSupport": "ACI",
                                "softReleaseMinVer": "ACI-N9KDK9-11.3(2)",
                                "networkDeviceNotes": None,
                                "releaseId": 1061,
                                "softReleaseDOM": "ACI-N9KDK9-11.3(2)",
                                "type": "Optic",
                            },
                            {
                                "tmgId": 24740,
                                "productFamilyId": 5,
                                "productFamily": "QSFP100",
                                "productModelId": 1,
                                "productId": "QSFP-100G-SR4-S",
                                "version": " ",
                                "versionId": None,
                                "description": None,
                                "formFactor": "QSFP28",
                                "reach": "100m (OM4)",
                                "temperatureRange": "0 to 70C",
                                "digitalDiagnostic": "Y",
                                "cableType": "Ribbon Fiber",
                                "media": "MMF",
                                "connectorType": "MPO-12",
                                "transmissionStandard": " ",
                                "transceiverModelDataSheet": "https://www.cisco.com/c/en/us/products/collateral/interfaces-modules/transceiver-modules/datasheet-c78-736282.html",
                                "endOfSale": " ",
                                "dataRate": "100 Gbps",
                                "transceiverNotes": None,
                                "noteCount": 0,
                                "state": None,
                                "stateMessage": None,
                                "updatedOn": None,
                                "updatedBy": None,
                                "transceiverBusinessUnit": "TMG",
                                "networkModelId": 43,
                                "breakoutMode": " ",
                                "osType": "NX-OS",
                                "domSupport": "NX-OS",
                                "softReleaseMinVer": "NX-OS 7.03I4.2",
                                "networkDeviceNotes": None,
                                "releaseId": 552,
                                "softReleaseDOM": "NX-OS 7.03I4.2",
                                "type": "Optic",
                            },
                        ],
                    }
                ],
            }
        ],
    }
    tmg_res = TMGResult.TMGResult(result_test_inputdata)
    assert tmg_res is not None
    assert tmg_res.result == result_test_inputdata
    assert tmg_res.total_count == result_test_inputdata["totalCount"]
    assert len(tmg_res.network_devices) == tmg_res.total_count
    net_dev = tmg_res.network_devices[0]
    assert net_dev.product_id == "N9K-C93180YC-EX"
    assert len(net_dev.transceivers) == 2
    first_xcvr = net_dev.transceivers[0]
    second_xcvr = net_dev.transceivers[1]
    assert first_xcvr.product_id == "QSFP-100G-SR4-S"
    assert second_xcvr.product_id == "QSFP-100G-SR4-S"



import pytest
import json
from pytmg.TMGTransceiver import TMGTransceiver


def test_tmg_transceiver_init_valid_data():
    input_data = """{
        "tmgId": 24739,
        "productFamilyId": 5,
        "productFamily": "QSFP100",
        "productModelId": 1,
        "productId": "QSFP-100G-SR4-S",
        "version": " ",
        "versionId": null,
        "description": null,
        "formFactor": "QSFP28",
        "reach": "100m (OM4)",
        "temperatureRange": "0 to 70C",
        "digitalDiagnostic": "Y",
        "cableType": "Ribbon Fiber",
        "media": "MMF",
        "connectorType": "MPO-12",
        "transmissionStandard": " ",
        "transceiverModelDataSheet": "https://www.cisco.com/c/en/us/products/collateral/interfaces-modules/transceiver-modules/datasheet-c78-736282.html",
        "endOfSale": " ",
        "dataRate": "100 Gbps",
        "transceiverNotes": null,
        "noteCount": 0,
        "state": null,
        "stateMessage": null,
        "updatedOn": null,
        "updatedBy": null,
        "transceiverBusinessUnit": "TMG",
        "networkModelId": 43,
        "breakoutMode": " ",
        "osType": "ACI",
        "domSupport": "ACI",
        "softReleaseMinVer": "ACI-N9KDK9-11.3(2)",
        "networkDeviceNotes": null,
        "releaseId": 1061,
        "softReleaseDOM": "ACI-N9KDK9-11.3(2)",
        "type": "Optic"
    }"""
    xcvr = TMGTransceiver(json.loads(input_data))
    assert xcvr is not None
    assert xcvr.tmg_id == 24739
    assert xcvr.product_family_id == 5
    assert xcvr.product_family == "QSFP100"
    assert xcvr.product_model_id == 1
    assert xcvr.product_id == "QSFP-100G-SR4-S"
    assert xcvr.version == " "
    assert xcvr.version_id is None
    assert xcvr.description is None
    assert xcvr.form_factor == "QSFP28"
    assert xcvr.reach == "100m (OM4)"
    assert xcvr.temperature_range == "0 to 70C"
    assert xcvr.digital_diagnostic == "Y"
    assert xcvr.cable_type == "Ribbon Fiber"
    assert xcvr.media == "MMF"
    assert xcvr.connector_type == "MPO-12"
    assert xcvr.transmission_standard == " "
    assert (
        xcvr.transceiver_model_data_sheet
        == "https://www.cisco.com/c/en/us/products/collateral/interfaces-modules/transceiver-modules/datasheet-c78-736282.html"
    )
    assert xcvr.end_of_sale == " "
    assert xcvr.data_rate == "100 Gbps"
    assert xcvr.transceiver_notes is None
    assert xcvr.note_count == 0
    assert xcvr.state is None
    assert xcvr.state_message is None
    assert xcvr.updated_on is None
    assert xcvr.updated_by is None
    assert xcvr.transceiver_business_unit == "TMG"
    assert xcvr.network_model_id == 43
    assert xcvr.breakout_mode == " "
    assert xcvr.os_type == "ACI"
    assert xcvr.dom_support == "ACI"
    assert xcvr.soft_release_min_ver == "ACI-N9KDK9-11.3(2)"
    assert xcvr.network_device_notes is None
    assert xcvr.release_id == 1061
    assert xcvr.soft_release_dom == "ACI-N9KDK9-11.3(2)"
    assert xcvr.type == "Optic"


def test_tmg_transceiver_clean_aci_11_sw_release():
    input_data = """{
        "tmgId": 24739,
        "productFamilyId": 5,
        "productFamily": "QSFP100",
        "productModelId": 1,
        "productId": "QSFP-100G-SR4-S",
        "version": " ",
        "versionId": null,
        "description": null,
        "formFactor": "QSFP28",
        "reach": "100m (OM4)",
        "temperatureRange": "0 to 70C",
        "digitalDiagnostic": "Y",
        "cableType": "Ribbon Fiber",
        "media": "MMF",
        "connectorType": "MPO-12",
        "transmissionStandard": " ",
        "transceiverModelDataSheet": "https://www.cisco.com/c/en/us/products/collateral/interfaces-modules/transceiver-modules/datasheet-c78-736282.html",
        "endOfSale": " ",
        "dataRate": "100 Gbps",
        "transceiverNotes": null,
        "noteCount": 0,
        "state": null,
        "stateMessage": null,
        "updatedOn": null,
        "updatedBy": null,
        "transceiverBusinessUnit": "TMG",
        "networkModelId": 43,
        "breakoutMode": " ",
        "osType": "ACI",
        "domSupport": "ACI",
        "softReleaseMinVer": "ACI-N9KDK9-11.3(2)",
        "networkDeviceNotes": null,
        "releaseId": 1061,
        "softReleaseDOM": "ACI-N9KDK9-11.3(2)",
        "type": "Optic"
    }"""
    xcvr = TMGTransceiver(json.loads(input_data))
    assert xcvr is not None
    assert xcvr.clean_soft_release_min_ver == "11.3(2)"


def test_tmg_transceiver_clean_aci_14_sw_release():
    input_data = """{
        "tmgId": 24739,
        "productFamilyId": 5,
        "productFamily": "QSFP100",
        "productModelId": 1,
        "productId": "QSFP-100G-SR4-S",
        "version": " ",
        "versionId": null,
        "description": null,
        "formFactor": "QSFP28",
        "reach": "100m (OM4)",
        "temperatureRange": "0 to 70C",
        "digitalDiagnostic": "Y",
        "cableType": "Ribbon Fiber",
        "media": "MMF",
        "connectorType": "MPO-12",
        "transmissionStandard": " ",
        "transceiverModelDataSheet": "https://www.cisco.com/c/en/us/products/collateral/interfaces-modules/transceiver-modules/datasheet-c78-736282.html",
        "endOfSale": " ",
        "dataRate": "100 Gbps",
        "transceiverNotes": null,
        "noteCount": 0,
        "state": null,
        "stateMessage": null,
        "updatedOn": null,
        "updatedBy": null,
        "transceiverBusinessUnit": "TMG",
        "networkModelId": 43,
        "breakoutMode": " ",
        "osType": "ACI",
        "domSupport": "ACI",
        "softReleaseMinVer": "ACI-N9KDK9-14.1(1)",
        "networkDeviceNotes": null,
        "releaseId": 1061,
        "softReleaseDOM": "ACI-N9KDK9-14.1(1)",
        "type": "Optic"
    }"""
    xcvr = TMGTransceiver(json.loads(input_data))
    assert xcvr is not None
    assert xcvr.clean_soft_release_min_ver == "14.1(1)"


def test_tmg_transceiver_clean_nxos_7_sw_release():
    input_data = """{
        "tmgId": 24740,
        "productFamilyId": 5,
        "productFamily": "QSFP100",
        "productModelId": 1,
        "productId": "QSFP-100G-SR4-S",
        "version": " ",
        "versionId": null,
        "description": null,
        "formFactor": "QSFP28",
        "reach": "100m (OM4)",
        "temperatureRange": "0 to 70C",
        "digitalDiagnostic": "Y",
        "cableType": "Ribbon Fiber",
        "media": "MMF",
        "connectorType": "MPO-12",
        "transmissionStandard": " ",
        "transceiverModelDataSheet": "https://www.cisco.com/c/en/us/products/collateral/interfaces-modules/transceiver-modules/datasheet-c78-736282.html",
        "endOfSale": " ",
        "dataRate": "100 Gbps",
        "transceiverNotes": null,
        "noteCount": 0,
        "state": null,
        "stateMessage": null,
        "updatedOn": null,
        "updatedBy": null,
        "transceiverBusinessUnit": "TMG",
        "networkModelId": 43,
        "breakoutMode": " ",
        "osType": "NX-OS",
        "domSupport": "NX-OS",
        "softReleaseMinVer": "NX-OS 7.03I4.2",
        "networkDeviceNotes": null,
        "releaseId": 552,
        "softReleaseDOM": "NX-OS 7.03I4.2",
        "type": "Optic"
    }"""
    xcvr = TMGTransceiver(json.loads(input_data))
    assert xcvr is not None
    assert xcvr.clean_soft_release_min_ver == "7.0(3)I4(2)"


def test_tmg_transceiver_clean_nxos_9_sw_release():
    input_data = """{
        "tmgId": 24740,
        "productFamilyId": 5,
        "productFamily": "QSFP100",
        "productModelId": 1,
        "productId": "QSFP-100G-SR4-S",
        "version": " ",
        "versionId": null,
        "description": null,
        "formFactor": "QSFP28",
        "reach": "100m (OM4)",
        "temperatureRange": "0 to 70C",
        "digitalDiagnostic": "Y",
        "cableType": "Ribbon Fiber",
        "media": "MMF",
        "connectorType": "MPO-12",
        "transmissionStandard": " ",
        "transceiverModelDataSheet": "https://www.cisco.com/c/en/us/products/collateral/interfaces-modules/transceiver-modules/datasheet-c78-736282.html",
        "endOfSale": " ",
        "dataRate": "100 Gbps",
        "transceiverNotes": null,
        "noteCount": 0,
        "state": null,
        "stateMessage": null,
        "updatedOn": null,
        "updatedBy": null,
        "transceiverBusinessUnit": "TMG",
        "networkModelId": 43,
        "breakoutMode": " ",
        "osType": "NX-OS",
        "domSupport": "NX-OS",
        "softReleaseMinVer": "NX-OS 9.2.1",
        "networkDeviceNotes": null,
        "releaseId": 552,
        "softReleaseDOM": "NX-OS 9.2.1",
        "type": "Optic"
    }"""
    xcvr = TMGTransceiver(json.loads(input_data))
    assert xcvr is not None
    assert xcvr.clean_soft_release_min_ver == "9.2(1)"



import pytest
from pytmg import TMGNetworkDevice


def test_tmg_network_device_init():
    test_product_family = "N9300"
    test_datasheet = "https://www.cisco.com/c/en/us/products/collateral/switches/nexus-9000-series-switches/datasheet-c78-741560.html"
    test_network_device_input = {
        "productId": "N9K-C93180YC-EX",
        "transceivers": [
            {
                "tmgId": 24739,
                "productFamilyId": 5,
                "productFamily": "QSFP100",
                "productModelId": 1,
                "productId": "QSFP-100G-SR4-S",
                "version": " ",
                "versionId": None,
                "description": None,
                "formFactor": "QSFP28",
                "reach": "100m (OM4)",
                "temperatureRange": "0 to 70C",
                "digitalDiagnostic": "Y",
                "cableType": "Ribbon Fiber",
                "media": "MMF",
                "connectorType": "MPO-12",
                "transmissionStandard": " ",
                "transceiverModelDataSheet": "https://www.cisco.com/c/en/us/products/collateral/interfaces-modules/transceiver-modules/datasheet-c78-736282.html",
                "endOfSale": " ",
                "dataRate": "100 Gbps",
                "transceiverNotes": None,
                "noteCount": 0,
                "state": None,
                "stateMessage": None,
                "updatedOn": None,
                "updatedBy": None,
                "transceiverBusinessUnit": "TMG",
                "networkModelId": 43,
                "breakoutMode": " ",
                "osType": "ACI",
                "domSupport": "ACI",
                "softReleaseMinVer": "ACI-N9KDK9-11.3(2)",
                "networkDeviceNotes": None,
                "releaseId": 1061,
                "softReleaseDOM": "ACI-N9KDK9-11.3(2)",
                "type": "Optic",
            },
            {
                "tmgId": 24740,
                "productFamilyId": 5,
                "productFamily": "QSFP100",
                "productModelId": 1,
                "productId": "QSFP-100G-SR4-S",
                "version": " ",
                "versionId": None,
                "description": None,
                "formFactor": "QSFP28",
                "reach": "100m (OM4)",
                "temperatureRange": "0 to 70C",
                "digitalDiagnostic": "Y",
                "cableType": "Ribbon Fiber",
                "media": "MMF",
                "connectorType": "MPO-12",
                "transmissionStandard": " ",
                "transceiverModelDataSheet": "https://www.cisco.com/c/en/us/products/collateral/interfaces-modules/transceiver-modules/datasheet-c78-736282.html",
                "endOfSale": " ",
                "dataRate": "100 Gbps",
                "transceiverNotes": None,
                "noteCount": 0,
                "state": None,
                "stateMessage": None,
                "updatedOn": None,
                "updatedBy": None,
                "transceiverBusinessUnit": "TMG",
                "networkModelId": 43,
                "breakoutMode": " ",
                "osType": "NX-OS",
                "domSupport": "NX-OS",
                "softReleaseMinVer": "NX-OS 7.03I4.2",
                "networkDeviceNotes": None,
                "releaseId": 552,
                "softReleaseDOM": "NX-OS 7.03I4.2",
                "type": "Optic",
            },
        ],
    }
    tmg_net_dev = TMGNetworkDevice.TMGNetworkDevice(
        test_network_device_input, test_product_family, test_datasheet
    )
    assert tmg_net_dev is not None
    assert tmg_net_dev.result == test_network_device_input
    assert tmg_net_dev.product_family == test_product_family
    assert tmg_net_dev.network_family_data_sheet == test_datasheet
    assert len(tmg_net_dev.transceivers) == 2
    first_xcvr = tmg_net_dev.transceivers[0]
    second_xcvr = tmg_net_dev.transceivers[1]
    assert first_xcvr.product_id == "QSFP-100G-SR4-S"
    assert second_xcvr.product_id == "QSFP-100G-SR4-S"



"""
Class representing a Cisco network device based upon results of Cisco TMG API
"""
from pytmg.TMGTransceiver import TMGTransceiver


class TMGNetworkDevice:
    """
    An object that represents a Cisco network device within the context of
    Cisco's TMG API.

    
    """

    def __init__(self, input, product_family, network_family_data_sheet):
        """
        Instantiates TMGNetworkDevice object and parses relevant results of Cisco's
        TMG API.

        The results of Cisco's TMG API are hierarchically organized so that each
        network device is directly associated with a list of supported
        transceivers. When searching in Cisco's TMG API, only transceivers and/or
        network devices that match the user-defined search parameters are returned
        as results.

        pyTMG's TMGNetworkDevice class mimics this hierarchy. The raw output of the
        relevant Cisco TMG API elements is accessible via the "result" attribute.
        The product family for the network device is accessible via the
        "product_family" attribute, and the relevant Cisco data sheet documentation
        is accessible via the "network_family_data_sheet" attribute.

        A list of supported transceivers that match the user-defined search
        parameters are accessible via a list in the "transceivers" attribute.
        Each object in this list is of the TMGTransceiver class.

        :param dict input: A dictionary containing the results of a TMG object 
            search against Cisco TMG's API relevant to a specific network
            device
        """
        self.result = input
        self.product_id = self.result["productId"]
        self.product_family = product_family
        self.network_family_data_sheet = network_family_data_sheet
        self.transceivers = []
        for transceiver in self.result["transceivers"]:
            self.transceivers.append(TMGTransceiver(transceiver))



"""
Class representing a Cisco-branded transceiver based upon results of Cisco TMG
API
"""
import re


class TMGTransceiver:
    """
    An object that represents a Cisco-branded transceiver within the context of
    Cisco's TMG API.

    """

    def __init__(self, input):
        """
        Instantiates TMGTransceiver object and parses relevant results of 
        Cisco's TMG API.

        Cisco's TMG API links specific PIDs/models of transceivers to network
        devices to indicate support. Furthermore, information about how a
        transceiver is supported (such as operating system version,
        environmental factors such as temperature range, etc.) is included.

        :param dict input: A dictionary containing the results of a TMG object 
            search against Cisco TMG's API relevant to a specific
            transceiver
        """
        self.result = input
        self.tmg_id = self.result["tmgId"]
        self.product_family_id = self.result["productFamilyId"]
        self.product_family = self.result["productFamily"]
        self.product_model_id = self.result["productModelId"]
        self.product_id = self.result["productId"]
        self.version = self.result["version"]
        self.version_id = self.result["versionId"]
        self.description = self.result["description"]
        self.form_factor = self.result["formFactor"]
        self.reach = self.result["reach"]
        self.temperature_range = self.result["temperatureRange"]
        self.digital_diagnostic = self.result["digitalDiagnostic"]
        self.cable_type = self.result["cableType"]
        self.media = self.result["media"]
        self.connector_type = self.result["connectorType"]
        self.transmission_standard = self.result["transmissionStandard"]
        self.transceiver_model_data_sheet = self.result["transceiverModelDataSheet"]
        self.end_of_sale = self.result["endOfSale"]
        self.data_rate = self.result["dataRate"]
        self.transceiver_notes = self.result["transceiverNotes"]
        self.note_count = self.result["noteCount"]
        self.state = self.result["state"]
        self.state_message = self.result["stateMessage"]
        self.updated_on = self.result["updatedOn"]
        self.updated_by = self.result["updatedBy"]
        self.transceiver_business_unit = self.result["transceiverBusinessUnit"]
        self.network_model_id = self.result["networkModelId"]
        self.breakout_mode = self.result["breakoutMode"]
        self.os_type = self.result["osType"]
        self.dom_support = self.result["domSupport"]
        self.soft_release_min_ver = self.result["softReleaseMinVer"]
        self.network_device_notes = self.result["networkDeviceNotes"]
        self.release_id = self.result["releaseId"]
        self.soft_release_dom = self.result["softReleaseDOM"]
        self.type = self.result["type"]
        self.clean_soft_release_min_ver = self._clean_soft_release()

    def _clean_soft_release(self):
        """
        Normalize network device software release provided by Cisco's TMG API.

        Cisco's TMG API sometimes presents network device software releases
        in a format that is inconsistent with the "official" format. The
        purpose of this method is to attempt to normalize the software
        release format presented by Cisco's TMG API to allow better
        interoperability.

        Currently, the below operating systems are supported:

        * Full Support:

        * Partial Support:
          - ACI
          - NX-OS
        """
        if self.os_type == "ACI":
            return self._clean_aci_soft_release()
        elif self.os_type == "NX-OS":
            return self._clean_nxos_soft_release()
        else:
            # Return "dirty" software release, which is better than nothing
            return self.soft_release_min_ver

    def _clean_aci_soft_release(self):
        """
        Normalize Cisco ACI network device software releases provided by
        Cisco's TMG API.

        Most Cisco-branded transceivers supported by ACI software return
        with a softReleaseMinVer format as follows:

            ACI-N9KDK9-11.3(2)

        The desired format is as follows:

            11.3(2)

        This method converts the first format into the second format.
        This is done by simply replacing the "ACI-N9KDK9-" substring in 
        the softReleaseMinVer value.
        """
        if "ACI-N9KDK9-" in self.soft_release_min_ver:
            return self.soft_release_min_ver.replace("ACI-N9KDK9-", "")

    def _clean_nxos_soft_release(self):
        """
        Normalize Cisco NX-OS network device software releases provided by
        Cisco's TMG API.

        Most Cisco-branded transceivers supported by NX-OS software return
        with a softReleaseMinVer format as follows:

            NX-OS 7.03I4.2

        The desired format is as follows:

            7.0(3)I4(2)

        This method converts the first format into the second format with 
        a regex string identifying:
        1. The major release (7.0(3)) (not part of a capture group)
        2. The minor release (I4) (capture group 1)
        3. The maintenance release (2) (capture group 2)

        Supported transceivers introduced in 9.x NX-OS software releases have
        a softReleaseMinVer format as follows:

            NX-OS 9.2.1

        The desired format is as follows:

            9.2(1)
        
        This method converts the first format into the second format with a
        regex string identifying:
        1. The minor release (2) (capture group 1)
        2. The maintenance release (1) (capture group 2)
        """
        if "NX-OS" in self.soft_release_min_ver:
            nxos_7_dirty_re = r"NX-OS 7.03I(\d+)\.(\d+)"
            nxos_7_res = re.search(nxos_7_dirty_re, self.soft_release_min_ver)
            if nxos_7_res:
                mnr_rel = nxos_7_res.group(1)
                mnt_rel = nxos_7_res.group(2)
                clean_sw_rel = "7.0(3)I{}({})".format(mnr_rel, mnt_rel)
                return clean_sw_rel
            nxos_9_dirty_re = r"NX-OS 9.(\d+)\.(\d+)"
            nxos_9_res = re.search(nxos_9_dirty_re, self.soft_release_min_ver)
            if nxos_9_res:
                mnr_rel = nxos_9_res.group(1)
                mnt_rel = nxos_9_res.group(2)
                clean_sw_rel = "9.{}({})".format(mnr_rel, mnt_rel)
                return clean_sw_rel



import pytest
import responses
import json
from pytmg import TMG
from pytmg import TMGResult
from .models import test_TMG_models


def test_tmg_init():
    tmg = TMG.TMG()
    assert tmg is not None


class TestPrivateSearchMethodSimple:
    @responses.activate
    def test_tmg_private_search_for_n9k_93180yc_ex(self):
        resp_json = test_TMG_models.N9K_C93180YC_EX_MODEL
        responses.add(
            responses.POST,
            "https://tmgmatrix.cisco.com/public/api/networkdevice/search",
            json=resp_json,
            status=200,
        )

        tmg = TMG.TMG()
        res = tmg._search(search_input=["N9K-C93180YC-EX"])
        assert res is not None
        assert (
            res["networkDevices"][0]["networkAndTransceiverCompatibility"][0][
                "productId"
            ]
            == "N9K-C93180YC-EX"
        )

    @responses.activate
    def test_tmg_private_search_for_n9k_9372px(self):
        resp_json = test_TMG_models.N9K_C9372PX_MODEL
        responses.add(
            responses.POST,
            "https://tmgmatrix.cisco.com/public/api/networkdevice/search",
            json=resp_json,
            status=200,
        )

        tmg = TMG.TMG()
        res = tmg._search(search_input=["N9K-C9372PX"])
        assert res is not None
        assert (
            res["networkDevices"][0]["networkAndTransceiverCompatibility"][0][
                "productId"
            ]
            == "N9K-C9372PX"
        )
        assert (
            res["networkDevices"][0]["networkAndTransceiverCompatibility"][1][
                "productId"
            ]
            == "N9K-C9372PX-E"
        )

    @responses.activate
    def test_tmg_private_search_for_ws_c3750_24ps(self):
        resp_json = test_TMG_models.WS_C3750_24PS_MODEL
        responses.add(
            responses.POST,
            "https://tmgmatrix.cisco.com/public/api/networkdevice/search",
            json=resp_json,
            status=200,
        )

        tmg = TMG.TMG()
        res = tmg._search(search_input=["WS-C3750-24PS"])
        assert res is not None
        assert (
            res["networkDevices"][0]["networkAndTransceiverCompatibility"][0][
                "productId"
            ]
            == "WS-C3750-24PS"
        )

    @responses.activate
    def test_tmg_private_search_for_all_3750s(self):
        resp_json = test_TMG_models.ALL_3750S_MODEL
        responses.add(
            responses.POST,
            "https://tmgmatrix.cisco.com/public/api/networkdevice/search",
            json=resp_json,
            status=200,
        )

        tmg = TMG.TMG()
        res = tmg._search(search_input=["WS-C3750"])
        assert res is not None
        product_list = [
            dev["productId"]
            for dev in res["networkDevices"][0]["networkAndTransceiverCompatibility"]
        ]
        product_list += [
            dev["productId"]
            for dev in res["networkDevices"][1]["networkAndTransceiverCompatibility"]
        ]
        assert "WS-C3750G-16TD" in product_list
        assert "WS-C3750-24PS" in product_list
        assert "WS-C3750-24TS" in product_list
        assert "WS-C3750-48PS" in product_list
        assert "WS-C3750-48TS" in product_list
        assert "WS-C3750G-12S" in product_list
        assert "WS-C3750G-24PS" in product_list
        assert "WS-C3750G-24TS" in product_list
        assert "WS-C3750G-24TS-E1U" in product_list
        assert "WS-C3750G-24TS-S1U" in product_list
        assert "WS-C3750G-48PS" in product_list
        assert "WS-C3750G-48TS" in product_list
        assert "WS-C3750-24FS-S" in product_list
        assert "WS-C3750V2-24PS" in product_list
        assert "WS-C3750V2-24TS" in product_list
        assert "WS-C3750V2-48PS" in product_list
        assert "WS-C3750V2-48TS" in product_list
        assert "WS-C3750V2-24FS-S" in product_list
        assert "WS-C3750E-24TD" in product_list
        assert "WS-C3750E-24PD" in product_list
        assert "WS-C3750E-48TD" in product_list
        assert "WS-C3750E-48PD" in product_list
        assert "WS-C3750E-48PDF" in product_list
        assert "WS-C3750X-12S" in product_list
        assert "WS-C3750X-24S" in product_list
        assert "WS-C3750-24WFS" in product_list
        assert "WS-C3750V2-24FS" in product_list
        assert "WS-C3750X-24T" in product_list
        assert "WS-C3750X-48T" in product_list
        assert "WS-C3750X-24P" in product_list
        assert "WS-C3750X-48P" in product_list
        assert "WS-C3750X-48PF" in product_list
        assert "WS-C3750X-12S" in product_list
        assert "WS-C3750X-24S" in product_list

    @responses.activate
    def test_tmg_private_search_for_xcvr_qsfp_40g_sr_bd(self):
        resp_json = test_TMG_models.XCVR_QSFP_40G_SR_BD
        responses.add(
            responses.POST,
            "https://tmgmatrix.cisco.com/public/api/networkdevice/search",
            json=resp_json,
            status=200,
        )

        tmg = TMG.TMG()
        res = tmg._search(search_input=["QSFP-40G-SR-BD"])
        assert res is not None
        assert res["totalCount"] == "33"
        assert res["networkDevices"] is not None
        assert len(res["networkDevices"]) == 1
        product_list = [
            dev["productId"]
            for dev in res["networkDevices"][0]["networkAndTransceiverCompatibility"]
        ]
        assert "C9400-SUP-1" in product_list
        assert "C9400-SUP-1XL" in product_list
        assert "C9400-SUP-1XL-Y" in product_list

        devices = res["networkDevices"][0]["networkAndTransceiverCompatibility"]
        for device in devices:
            for xcvr in device["transceivers"]:
                assert xcvr["productId"] == "QSFP-40G-SR-BD"


class TestDeviceSearchingSimple:
    @responses.activate
    def test_tmg_search_for_n9k_93180yc_ex(self):
        resp_json = test_TMG_models.N9K_C93180YC_EX_MODEL
        responses.add(
            responses.POST,
            "https://tmgmatrix.cisco.com/public/api/networkdevice/search",
            json=resp_json,
            status=200,
        )

        tmg = TMG.TMG()
        res = tmg.search_device("N9K-C93180YC-EX")
        assert res is not None
        assert res.network_devices[0].product_id == "N9K-C93180YC-EX"

    @responses.activate
    def test_tmg_search_for_n9k_9372px(self):
        resp_json = test_TMG_models.N9K_C9372PX_MODEL
        responses.add(
            responses.POST,
            "https://tmgmatrix.cisco.com/public/api/networkdevice/search",
            json=resp_json,
            status=200,
        )

        tmg = TMG.TMG()
        res = tmg.search_device("N9K-C9372PX")
        assert res is not None
        assert res.network_devices[0].product_id == "N9K-C9372PX"

    @responses.activate
    def test_tmg_search_for_ws_c3750_24ps(self):
        resp_json = test_TMG_models.WS_C3750_24PS_MODEL
        responses.add(
            responses.POST,
            "https://tmgmatrix.cisco.com/public/api/networkdevice/search",
            json=resp_json,
            status=200,
        )

        tmg = TMG.TMG()
        res = tmg.search_device("WS-C3750-24PS")
        assert res is not None
        assert res.network_devices[0].product_id == "WS-C3750-24PS"

    @responses.activate
    def test_tmg_search_all_3750s(self):
        resp_json = test_TMG_models.ALL_3750S_MODEL
        responses.add(
            responses.POST,
            "https://tmgmatrix.cisco.com/public/api/networkdevice/search",
            json=resp_json,
            status=200,
        )

        tmg = TMG.TMG()
        res = tmg.search_device("WS-C3750")
        assert res is not None
        assert len(res.network_devices) > 1
        # Verify expected products are in returned list
        product_list = [dev.product_id for dev in res.network_devices]
        assert "WS-C3750G-16TD" in product_list
        assert "WS-C3750-24PS" in product_list
        assert "WS-C3750-24TS" in product_list
        assert "WS-C3750-48PS" in product_list
        assert "WS-C3750-48TS" in product_list
        assert "WS-C3750G-12S" in product_list
        assert "WS-C3750G-24PS" in product_list
        assert "WS-C3750G-24TS" in product_list
        assert "WS-C3750G-24TS-E1U" in product_list
        assert "WS-C3750G-24TS-S1U" in product_list
        assert "WS-C3750G-48PS" in product_list
        assert "WS-C3750G-48TS" in product_list
        assert "WS-C3750-24FS-S" in product_list
        assert "WS-C3750V2-24PS" in product_list
        assert "WS-C3750V2-24TS" in product_list
        assert "WS-C3750V2-48PS" in product_list
        assert "WS-C3750V2-48TS" in product_list
        assert "WS-C3750V2-24FS-S" in product_list
        assert "WS-C3750E-24TD" in product_list
        assert "WS-C3750E-24PD" in product_list
        assert "WS-C3750E-48TD" in product_list
        assert "WS-C3750E-48PD" in product_list
        assert "WS-C3750E-48PDF" in product_list
        assert "WS-C3750X-12S" in product_list
        assert "WS-C3750X-24S" in product_list
        assert "WS-C3750-24WFS" in product_list
        assert "WS-C3750V2-24FS" in product_list
        assert "WS-C3750X-24T" in product_list
        assert "WS-C3750X-48T" in product_list
        assert "WS-C3750X-24P" in product_list
        assert "WS-C3750X-48P" in product_list
        assert "WS-C3750X-48PF" in product_list
        assert "WS-C3750X-12S" in product_list
        assert "WS-C3750X-24S" in product_list

    @responses.activate
    def test_tmg_search_xcvr_qsfp_40g_sr_bd(self):
        resp_json = test_TMG_models.XCVR_QSFP_40G_SR_BD
        responses.add(
            responses.POST,
            "https://tmgmatrix.cisco.com/public/api/networkdevice/search",
            json=resp_json,
            status=200,
        )

        tmg = TMG.TMG()
        res = tmg.search_device("QSFP-40G-SR-BD")
        assert res is not None
        assert len(res.network_devices) == 3
        assert res.network_devices[0].product_id == "C9400-SUP-1"
        assert res.network_devices[1].product_id == "C9400-SUP-1XL"
        assert res.network_devices[2].product_id == "C9400-SUP-1XL-Y"
        for device in res.network_devices:
            for xcvr in device.transceivers:
                assert xcvr.product_id == "QSFP-40G-SR-BD"

    @responses.activate
    def test_tmg_search_multiple_devices(self):
        def tmg_callback(request):
            payload = json.loads(request.body)
            if payload["searchInput"][0] == "N9K-C93180YC-FX":
                resp_json = test_TMG_models.N9K_C93180YC_FX_MODEL
                return (
                    200,
                    {"content-type": "application/json"},
                    json.dumps(resp_json),
                )
            elif payload["searchInput"][0] == "C9300-48S":
                resp_json = test_TMG_models.C9300_48S_MODEL
                return (
                    200,
                    {"content-type": "application/json"},
                    json.dumps(resp_json),
                )
            elif payload["searchInput"][0] == "2951":
                resp_json = test_TMG_models.ISR2951_MODEL
                return (
                    200,
                    {"content-type": "application/json"},
                    json.dumps(resp_json),
                )
            else:
                return (503, {}, None)

        responses.add_callback(
            responses.POST,
            "https://tmgmatrix.cisco.com/public/api/networkdevice/search",
            callback=tmg_callback,
            content_type="application/json",
        )

        device_list = [
            "N9K-C93180YC-FX",  # Nexus 93180YC-FX
            "C9300-48S",  # Catalyst 9300-48S
            "2951",  # ISR 2951
        ]
        tmg = TMG.TMG()
        res_list = tmg.search_devices(device_list)
        assert res_list is not None
        assert len(res_list) == 3
        for result in res_list:
            for result_device in result.network_devices:
                assert result_device.product_id in device_list


class TestPrivateSearchMethodAdvanced:
    @responses.activate
    def test_tmg_private_search_advanced_ios_xe_fet_10g(self):
        resp_json = test_TMG_models.ADV_SEARCH_IOS_XE_FET_10G_MODEL
        responses.add(
            responses.POST,
            "https://tmgmatrix.cisco.com/public/api/networkdevice/search",
            json=resp_json,
            status=200,
        )

        tmg = TMG.TMG()
        res = tmg._search(
            cable_type=["Duplex Fiber"],
            data_rate=["10 Gbps"],
            form_factor=["SFP+"],
            reach=["100m"],
            os_type=["IOS XE"],
        )
        assert res is not None
        assert int(res["totalCount"]) >= 2
        devices = res["networkDevices"]

        # Verify product families
        assert devices[0]["productFamily"] == "C9300"
        assert devices[1]["productFamily"] == "C9300L"

        # Verify device PIDs
        dev_one = devices[0]["networkAndTransceiverCompatibility"][0]
        dev_two = devices[1]["networkAndTransceiverCompatibility"][0]
        dev_three = devices[1]["networkAndTransceiverCompatibility"][1]
        dev_four = devices[1]["networkAndTransceiverCompatibility"][2]
        dev_five = devices[1]["networkAndTransceiverCompatibility"][3]
        assert dev_one["productId"] == "C9300-NM-8X"
        assert dev_two["productId"] == "C9300L-24T-4X"
        assert dev_three["productId"] == "C9300L-48T-4X"
        assert dev_four["productId"] == "C9300L-24P-4X"
        assert dev_five["productId"] == "C9300L-48P-4X"

        # Verify each device's transceivers
        dev_one["transceivers"][0]["productId"] == "FET-10G"
        dev_two["transceivers"][0]["productId"] == "FET-10G"
        dev_three["transceivers"][0]["productId"] == "FET-10G"
        dev_four["transceivers"][0]["productId"] == "FET-10G"
        dev_five["transceivers"][0]["productId"] == "FET-10G"

    @responses.activate
    def test_tmg_private_search_advanced_ucs_xr_1gbps(self):
        resp_json = test_TMG_models.ADV_SEARCH_UCS_XR_1GBPS_MODEL
        responses.add(
            responses.POST,
            "https://tmgmatrix.cisco.com/public/api/networkdevice/search",
            json=resp_json,
            status=200,
        )

        tmg = TMG.TMG()
        res = tmg._search(
            cable_type=["Cat5e/6A"],
            data_rate=["10/100/1000 Mbps"],
            form_factor=["SFP"],
            reach=["100m"],
            os_type=["IOS XR", "UCS"],
        )
        assert res is not None
        assert int(res["totalCount"]) >= 5
        devices = res["networkDevices"]

        # Verify product families
        assert devices[0]["productFamily"] == "UCSB"
        assert devices[1]["productFamily"] == "NCS5000"
        assert devices[2]["productFamily"] == "NCS5500"
        assert devices[3]["productFamily"] == "ASR9000"
        assert devices[4]["productFamily"] == "NCS540"

        # Verify device PIDs
        expected_devices = [
            "UCS-FI-M-6324",
            "UCS-FI-6332-U",
            "UCS-FI-6332-16UP-U",
            "N10-S6100",
            "N10-S6200",
            "N10-E0440",
            "N10-E0600",
            "N20-I6584",
            "UCS-FI-6248UP",
            "UCS-FI-E16UP",
            "UCS-IOM-2208XP",
            "UCS-FI-6454",
            "NCS 5001",
            "NCS 5002",
            "NC55-24H12F-SE",
            "NCS-5501",
            "NCS-5501-SE",
            "NC55-MOD-A-S",
            "NCS-55A2-MOD-S",
            "NC55-MOD-A-SE-S",
            "NCS-55A2-MOD-HD-S",
            "NCS-55A2-MOD-SE-S",
            "NCS-55A2-MOD-HX-S",
            "NCS-55A2-MOD-SE-H-S",
            "A9K-40GE-B",
            "A9K-40GE-E",
            "A9K-2T20GE-B",
            "A9K-2T20GE-E",
            "A9K-2T20GE-L",
            "A9K-40GE-L",
            "A9K-MPA-20X1GE",
            "A9K-40GE-TR",
            "A9K-40GE-SE",
            "A9K-4T16GE-TR",
            "A9K-4T16GE-SE",
            "A9K-RSP440-TR",
            "A9K-RSP440-SE",
            "ASR-9000V-AC",
            "ASR-9000V-DC-A",
            "ASR-9000V-DC-E",
            "ASR-9000V-24-A",
            "A9KV-V2-AC",
            "A9KV-V2-DC-A",
            "A9KV-V2-DC-E",
            "A9K-24X10GE-1G-SE",
            "A9K-24X10GE-1G-TR",
            "A9K-48X10GE-1G-SE",
            "A9K-48X10GE-1G-TR",
            "A9K-RSP440-LT",
            "A99-48X10GE-1G-SE",
            "A99-48X10GE-1G-TR",
            "N540-24Z8Q2C-SYS",
            "N540-ACC-SYS",
            "N540X-ACC-SYS",
            "N540-24Z8Q2C-M",
        ]
        for pf in devices:
            for dev in pf["networkAndTransceiverCompatibility"]:
                assert dev["productId"] in expected_devices

        # Verify supported transceivers
        for pf in devices:
            for dev in pf["networkAndTransceiverCompatibility"]:
                for xcvr in dev["transceivers"]:
                    assert xcvr["cableType"] == "Cat5e/6A"
                    assert xcvr["dataRate"] == "10/100/1000 Mbps"
                    assert xcvr["reach"] == "100m"
                    assert xcvr["formFactor"] == "SFP"
                    assert xcvr["osType"] == "UCS" or xcvr["osType"] == "IOS XR"
                    assert (
                        xcvr["productId"] == "GLC-T"
                        or xcvr["productId"] == "GLC-TE"
                        or xcvr["productId"] == "SFP-GE-T"
                    )


class TestDeviceSearchingAdvanced:
    @responses.activate
    def test_tmg_search_advanced_ios_xe_fet_10g(self):
        resp_json = test_TMG_models.ADV_SEARCH_IOS_XE_FET_10G_MODEL
        responses.add(
            responses.POST,
            "https://tmgmatrix.cisco.com/public/api/networkdevice/search",
            json=resp_json,
            status=200,
        )

        tmg = TMG.TMG()
        params = {
            "cable_type": ["Duplex Fiber"],
            "data_rate": ["10 Gbps"],
            "form_factor": ["SFP+"],
            "reach": ["100m"],
            "os_type": ["IOS XE"],
        }
        res = tmg.search(**params)
        assert res is not None
        assert res.total_count >= 2

        # Verify product families
        assert res.network_devices[0].product_family == "C9300"
        assert res.network_devices[1].product_family == "C9300L"

        # Verify device PIDs
        assert res.network_devices[0].product_id == "C9300-NM-8X"
        assert res.network_devices[1].product_id == "C9300L-24T-4X"
        assert res.network_devices[2].product_id == "C9300L-48T-4X"
        assert res.network_devices[3].product_id == "C9300L-24P-4X"
        assert res.network_devices[4].product_id == "C9300L-48P-4X"

        # Verify each device's transceivers
        assert res.network_devices[0].transceivers[0].product_id == "FET-10G"
        assert res.network_devices[1].transceivers[0].product_id == "FET-10G"
        assert res.network_devices[2].transceivers[0].product_id == "FET-10G"
        assert res.network_devices[3].transceivers[0].product_id == "FET-10G"
        assert res.network_devices[4].transceivers[0].product_id == "FET-10G"



"""
Class for results of calls to pyTMG TMG object
"""
from pytmg.TMGNetworkDevice import TMGNetworkDevice


class TMGResult:
    """
    The TMGResult object is the primary "exit point" for the pyTMG library.


    """

    def __init__(self, input_results):
        """
        Instantiates TMGResult object and parses results of Cisco's TMG API.

        The results of Cisco's TMG API are hierarchically organized so that
        network devices are at the "top" of the tree, while the transceivers
        supported on those network devices are attached to each network
        device at the bottom of the tree. For example, consider a search
        for the below devices:
        
        * N9K-C93180YC-EX
        * N9K-C9372PX
        * N9K-C9336C-FX2

        A QSFP-40G-SR4 transceiver is supported for the NX-OS operating
        system on all three of these network devices. Cisco's TMG API
        would return results structured as follows:

        Results
        +-- Network Devices
            +-- N9K-C93180YC-EX
                +-- QSFP-40G-SR
            +-- N9K-C9372PX
                +-- QSFP-40G-SR
            +-- N9K-C9336C-FX2
                +-- QSFP-40G-SR
        
        pyTMG's TMGResult class mimics this hierarchy. The full output of
        Cisco's TMG API is accessible via the "result" attribute. The
        total number of network devices contained in the results is accessible
        via the "total_count" attribute. The "network_devices" attribute
        contains a list of network devices that match the search parameters.

        Each network device in the "network_devices" list attribute is an
        object of TMGNetworkDevice. The hierarchy of transceivers that
        match the search parameters are organized under each TMGNetworkDevice
        object.

        :param dict input_results: A dictionary containing the results of a TMG
            object's search against Cisco's TMG API.
        """
        self.result = input_results
        self.total_count = int(self.result["totalCount"])
        self.network_devices = []
        for product_family in self.result["networkDevices"]:
            for network_device in product_family["networkAndTransceiverCompatibility"]:
                self.network_devices.append(
                    TMGNetworkDevice(
                        network_device,
                        product_family["productFamily"],
                        product_family["networkFamilyDataSheet"],
                    )
                )
